package it.eng.spagobi.presentation.publishers;

import java.util.Collection;

import it.eng.spago.base.RequestContainer;
import it.eng.spago.base.ResponseContainer;
import it.eng.spago.base.SourceBean;
import it.eng.spago.base.SourceBeanException;
import it.eng.spago.error.EMFErrorHandler;
import it.eng.spago.error.EMFUserError;
import it.eng.spagobi.constants.SpagoBIConstants;
import junit.framework.TestCase;

public class DetailModalitiesValuePublisherJUnitTest extends TestCase {

	private SourceBean request = null, response = null;
	
	private RequestContainer reqContainer = null;

	private ResponseContainer resContainer = null;
	
	private DetailModalitiesValuePublisher detailModalitiesValuePublisher = null;

	protected void setUp() throws Exception {
		try {
			request = new SourceBean("");
			response = new SourceBean("");
		} catch (SourceBeanException e1) {
			e1.printStackTrace();
			fail("Unaspected exception occurred!");
		}
		reqContainer = new RequestContainer();
		resContainer = new ResponseContainer();
		reqContainer.setServiceRequest(request);
		resContainer.setServiceResponse(response);
		resContainer.setErrorHandler(new EMFErrorHandler());
		detailModalitiesValuePublisher = new DetailModalitiesValuePublisher();
		super.setUp();
	}
	
	/**
	 * Test method for 'it.eng.spagobi.presentation.publishers.DetailModalitiesValuePublisher.getPublisherName(RequestContainer, ResponseContainer)'.
	 * Verifies that the correct Publisher name is returned.
	 */
	public void testGetPublisherName() {
		String publisherName = null;
		SourceBean moduleResponse = null;
		String moduleName = "DetailModalitiesValueModule";
		try {
			moduleResponse = new SourceBean(moduleName);
			moduleResponse.setAttribute("loopback","");
			response.setAttribute(moduleResponse);
			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
			assertEquals("detailModalitiesValueLoop",publisherName);
			
			moduleResponse.delAttribute("loopback");
			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
			assertEquals("detailModalitiesValue",publisherName);
			
			moduleResponse.setAttribute("runtest","");
			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
			assertEquals("detailLovTestResult",publisherName);
			
//			moduleResponse.setAttribute(SpagoBIConstants.WIZARD,SpagoBIConstants.WIZARD_QUERY);
//			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
//			assertEquals("querySelectionWizard",publisherName);
//			
//			moduleResponse.updAttribute(SpagoBIConstants.WIZARD,SpagoBIConstants.WIZARD_FIX_LOV);
//			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
//			assertEquals("lovSelectionWizard",publisherName);
//			
//			moduleResponse.updAttribute(SpagoBIConstants.WIZARD,SpagoBIConstants.WIZARD_SCRIPT);
//			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
//			assertEquals("scriptSelectionWizard",publisherName);
			
			response.delAttribute(moduleName);
			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
			assertEquals("error",publisherName);
			Collection errors = resContainer.getErrorHandler().getErrors();
			assertEquals(1, errors.size());
			assertEquals(10, ((EMFUserError)errors.toArray()[0]).getCode());
			
			publisherName = detailModalitiesValuePublisher.getPublisherName(reqContainer,resContainer);
			assertEquals("detailModalitiesValue",publisherName);
			
		} catch (SourceBeanException e) {
			e.printStackTrace();
			fail("Unaspected exception occurred!");
		}
	}

}
